#
# Report xarxa neural: dades provinents de PROBEN1
#

library("genann")
library("xtable")
options(xtable.only.contents = TRUE)  # no inclure l'entorn "tabuar" en la sortida
options(xtable.sanitize.colnames.function = identity)  # no convertir strings

# produir sempre el mateix resultat
set.seed(2016)


report_proben1 <- function(orig, train, test, inputs, outputs)
{
    train_mse_vec <- NULL
    test_mse_vec <- NULL

    for (i in 1:10)
    {
        model <- ann(train[inputs], train[outputs], hls=c(8,8), lact="sigmoid", lr=0.1, mo=0.5, tmse=0, maxep=1000)

        fitted <- predict(model, newdata=train[inputs])
        errors <- train[outputs] - fitted
        train_mse = sum(errors^2) / (nrow(errors) * ncol(errors))  # normalitzar segons #exemples i #sortides

        fitted <- predict(model, newdata=test[inputs])
        errors <- test[outputs] - fitted
        test_mse = sum(errors^2) / (nrow(errors) * ncol(errors))
        
        train_mse_vec <- c(train_mse_vec, train_mse)
        test_mse_vec  <- c(test_mse_vec, test_mse)
    }
    
    r = list(
        mean(train_mse_vec) * 100,
        sd(train_mse_vec) * 100,
        mean(test_mse_vec) * 100,
        sd(test_mse_vec) * 100
        )
    
    return(r)
}


b1 <- report_proben1(building1, building1_train, building1_test, 1:14, 15:17)
b2 <- report_proben1(building2, building2_train, building2_test, 1:14, 15:17)
b3 <- report_proben1(building3, building3_train, building3_test, 1:14, 15:17)

f1 <- report_proben1(flare1, flare1_train, flare1_test, 1:24, 25:27)
f2 <- report_proben1(flare2, flare2_train, flare2_test, 1:24, 25:27)
f3 <- report_proben1(flare3, flare3_train, flare3_test, 1:24, 25:27)

c1 <- report_proben1(card1, card1_train, card1_test, 1:51, 52:53)
c2 <- report_proben1(card2, card2_train, card2_test, 1:51, 52:53)
c3 <- report_proben1(card3, card3_train, card3_test, 1:51, 52:53)

h1 <- report_proben1(heart1, heart1_train, heart1_test, 1:35, 36:37)
h2 <- report_proben1(heart2, heart2_train, heart2_test, 1:35, 36:37)
h3 <- report_proben1(heart3, heart3_train, heart3_test, 1:35, 36:37)

t1 <- report_proben1(thyroid1, thyroid1_train, thyroid1_test, 1:21, 22:24)
t2 <- report_proben1(thyroid2, thyroid2_train, thyroid2_test, 1:21, 22:24)
t3 <- report_proben1(thyroid3, thyroid3_train, thyroid3_test, 1:21, 22:24)


results <- rbind(
	"building1" = b1,
	"building2" = b2,
	"building3" = b3,
	"flare1"    = f1,
	"flare2"    = f2,
	"flare3"    = f3,
	"card1"     = c1,
	"card2"     = c2,
	"card3"     = c3,
	"heart1"    = h1,
	"heart2"    = h2,
	"heart3"    = h3,
	"thyroid1"  = t1,
	"thyroid2"  = t2,
	"thyroid3"  = t3
	)


results <- as.data.frame(results)
colnames(results) <- c("$\\bar x$", "$s$", "$\\bar x$", "$s$")

results_xtab <- xtable(results, digits=c(0,3,3,3,3))

print(results)
print(results_xtab, file="report/ann_proben1.tex")


