#
# Prova amb funció d'activació lineal i XN saturada
#

#install.packages("../genann", repos=NULL)
library("genann")

set.seed(2016)


### Amb "tanh" no es pot aprendre el model, xq la sortida està fora de [-1,1]:
#model <- ann(satfunc_train$x, satfunc_train$y, lact="tanh", hls=c(30, 60, 30))

### Amb "linear" si:
#model <- ann(satfunc_train$x, satfunc_train$y, lact="linear", hls=c(30, 60, 30))

### L'AG pot trobar la XN adient, però es trobarà moltes amb saturació,
### que es detecten i penalitzen correctament:
evol <- gann(satfunc_train$x, satfunc_train$y, satfunc_val$x, satfunc_val$y, lact="linear")
model <- ann(satfunc_train$x, satfunc_train$y, lact="linear", tmse=evol$besttmse, lr=evol$bestlr, mo=evol$bestmo, hls=evol$besthls)


# ajustar dades noves i validar el model
fitted <- predict(model, newdata=satfunc_test$x)  # realitzar prediccions amb el conjunt test
errors <- satfunc_test$y - fitted                 # residus
print(sum(errors^2) / nrow(errors))               # error quadràtic mig

# generar gràfics per comprobar el model
pdf("report/gann_test_saturated.pdf", width=7, height=7)
par(mfrow=c(3, 2))
par(mar=c(3, 3, 2, 1), mgp=c(2, 0.7, 0), bty="n")

plot(evol, main="Evolució de l'AG", xlab="generació", ylab="fitness")
plot.ts(model$errors, log="y", main="Entrenament de la XN", xlab="epochs", ylab="EQM")

plot(satfunc_train$x, satfunc_train$y, ylim=c(-3, 4), col=rgb(0,0,0,0.4), pch=16,
     main="Conjunt d'entrenament", xlab="x d'entrenament", ylab="y d'entrenament",
     panel.first=grid(col="grey"))

plot(satfunc_test$x, fitted, ylim=c(-3, 4), col=rgb(0,0,0,0.4), pch=16,
     main="Prediccions", xlab="x de prova", ylab="prediccions",
     panel.first=grid(col="grey"))

hist(errors, main="Histograma de residus", xlab="error de predicció", ylab="freqüència")

plot(satfunc_test$x, errors, ylim=c(-3, 4), col=rgb(0,0,0,0.4), pch=16,
     main="Residus vs x", xlab="x de prova", ylab="error de predicció",
     panel.first=grid(col="grey"))
abline(0, 0)

dev.off()


