#
# Formateja i guarda els resultats de l'ajust de l'AG
#

library("xtable")
options(xtable.only.contents = TRUE)  # no inclure l'entorn "tabuar" en la sortida
options(xtable.sanitize.colnames.function = identity)  # no convertir strings


load_observation <- function(trainset, of, el, sb, cp, mp)
{
    datasetname = deparse(substitute(trainset))
    fname = sprintf("gann_tunning_data/%s_%d_%d_%d_%3.2f_%3.2f.Rdata", datasetname, of, el, sb, cp, mp)

    #try(load(fname), silent=TRUE)
    #if (exists("obs")) return(obs)
    #else return(NULL)
    
    load(fname)
    print(obs)
    
    return(obs)
}

load_sampling <- function(of=16, el=3, sb=4, cp=0.95, mp=0.05)
{
    p1 <- load_observation(sinfunc_train,   of, el, sb, cp, mp)
    p2 <- load_observation(discfunc_train,  of, el, sb, cp, mp)
    p3 <- load_observation(building1_train, of, el, sb, cp, mp)
    p4 <- load_observation(flare1_train,    of, el, sb, cp, mp)
    p5 <- load_observation(card1_train,     of, el, sb, cp, mp)
    p6 <- load_observation(heart1_train,    of, el, sb, cp, mp)
    p7 <- load_observation(thyroid1_train,  of, el, sb, cp, mp)

    fp1 = p1[[7]]
    fp2 = p2[[7]]
    fp3 = p3[[7]]
    fp4 = p4[[7]]
    fp5 = p5[[7]]
    fp6 = p6[[7]]
    fp7 = p7[[7]]

    lp1 = p1[[8]]
    lp2 = p2[[8]]
    lp3 = p3[[8]]
    lp4 = p4[[8]]
    lp5 = p5[[8]]
    lp6 = p6[[8]]
    lp7 = p7[[8]]

    fpmean = mean(c(fp1, fp2, fp3, fp4, fp5, fp6, fp7))
    lpmean = mean(c(lp1, lp2, lp3, lp4, lp5, lp6, lp7))

    fmatrix <<- rbind(fmatrix, c(of, el, sb, cp, mp, fp1, fp2, fp3, fp4, fp5, fp6, fp7, fpmean))
    lmatrix <<- rbind(lmatrix, c(of, el, sb, cp, mp, lp1, lp2, lp3, lp4, lp5, lp6, lp7, lpmean))
}


fmatrix <- matrix(ncol=13, nrow=0)
lmatrix <- matrix(ncol=13, nrow=0)


##### Carregar les dades guardades

# paràmetres base
load_sampling()

# variant offspring
load_sampling(of=8)
load_sampling(of=32)

# variant elitisme
load_sampling(el=1)
load_sampling(el=9)

# variant pressió de selecció
load_sampling(sb=2)
load_sampling(sb=8)

# variant probabilitat de creuament
load_sampling(cp=0.90)
load_sampling(cp=0.99)

# variant probabilitat de mutació
load_sampling(mp=0.01)
load_sampling(mp=0.10)

# variants addicionals, 2a iteració de la optimització
load_sampling(of=24, el=2, sb=6, cp=0.99, mp=0.07)
load_sampling(of=24, el=2, sb=8, cp=0.99, mp=0.07)
load_sampling(of=24, el=2, sb=6, cp=0.99, mp=0.05)
load_sampling(of=24, el=2, sb=6, cp=0.99, mp=0.09)
load_sampling(of=24, el=3, sb=6, cp=0.99, mp=0.07)
load_sampling(of=24, el=5, sb=6, cp=0.99, mp=0.07)

#####


print(fmatrix)
print(lmatrix)

fresults <- as.data.frame(fmatrix)
lresults <- as.data.frame(lmatrix)

#names <- c("D", "E", "S", "C", "M", "1", "2", "3", "4", "5", "6", "7", "$\\bar x$")
names <- c("Descendència", "Elitisme", "Selecció", "Creuament", "Mutació",
           "sinfunc", "discfunc", "building1", "flare1", "card1", "heart1", "thyroid1",
           "mitja")
colnames(fresults) <- names
colnames(lresults) <- names

fresults_m <- fresults
fresults_m[,6:13] <- fresults_m[,6:13] * 1000  # multiplicar per 1000 columnes 6:13 per major claredat en la taula
fresults_xtab <- xtable(fresults_m, digits=c(0, 0,0,0,2,2, 1,1,1,1,1,1,1, 1))
lresults_xtab <- xtable(lresults,   digits=c(0, 0,0,0,2,2, 0,0,0,0,0,0,0, 0))

print(fresults_xtab, include.rownames=TRUE, rotate.colnames=TRUE, file="report/gann_tunning_fmatrix.tex")
print(lresults_xtab, include.rownames=TRUE, rotate.colnames=TRUE, file="report/gann_tunning_lmatrix.tex")

# extreure les columnes de mitjes
f_means <- rev(fresults)[1]  # les mitjes són l'última columna del dataframe
l_means <- rev(lresults)[1]

mat <- data.frame(l_means, f_means)
rnk <- paretorank(as.matrix(mat))
pnd <- mat[rnk==0, ]  # Pareto non-dominated

# gràfic del pareto-rank
pdf("report/gann_tunning_pareto.pdf", width=7, height=5.5)
par(mar=c(3, 3, 2, 1), mgp=c(2, 0.7, 0), bty="n")

plot(mat, type="n",
     main="Pareto front", xlab="lifespans", ylab="fitness",
     panel.first=grid(col="grey"))
     
lines(pnd[order(pnd[,1]),], col="gray")  # dibuixar el Pareto-front
#text(mat, labels=rnk)
#text(jitter(as.matrix(mat)), labels=1:nrow(mat))
text(mat, labels=1:nrow(mat), cex=0.9)

dev.off()

# taula de dades
mat$rnk <- rnk  # afegir el pareto-rank a la matriu de mitges
colnames(mat) <- c("lifespans", "fitness", "rank")
mat_xtab <- xtable(mat, digits=c(0, 0,6,0))
print(mat_xtab, include.rownames=TRUE, file="report/gann_tunning_pareto.tex")



