################################################################################
# Marc de treball per proves i reports.
# Copyright (C) 2016 Jordi Mas Mateo <jordimas@uoc.edu>
################################################################################

#install.packages("xtable", dependencies=TRUE)
#install.packages("xtable", dependencies=FALSE)
install.packages("../genann", repos=NULL)


################################################################################
# Generar datasets sintètics
################################################################################

set.seed(2016)  # intentar produir les mateixes dades

# operacions lògiques, en aquests casos no ha de generalitzar, només aprendre el model
andfunc <- data.frame(p=c(0, 0, 1, 1), q=c(0, 1, 0, 1), r=c(0, 0, 0, 1))
xorfunc <- data.frame(p=c(0, 0, 1, 1), q=c(0, 1, 0, 1), r=c(0, 1, 1, 0))
#

# funcions deterministes, en aquest cas ha de generalitzar
sinfunc <- data.frame(x=seq(-1, 1, by=0.01))
sinfunc$y <- sin(pi * sinfunc$x)
sinfunc <- sinfunc[sample(nrow(sinfunc)),]
sinfunc_train <- sinfunc[1:100,]
sinfunc_val   <- sinfunc[101:150,]
sinfunc_test  <- sinfunc[151:201,]

discfunc <- data.frame(x=c(seq(-1, -0.1, by=0.01), seq(0.1, 1, by=0.01)))  # discontinua en x=0
discfunc$y <- 0.1 / discfunc$x
discfunc <- discfunc[sample(nrow(discfunc)),]
discfunc_train <- discfunc[1:100,]
discfunc_val   <- discfunc[101:140,]
discfunc_test  <- discfunc[141:182,]
#

# funció soroll, no es pot generalitzar ni aproximar
noisefunc <- data.frame(x=seq(-1, 1, by=0.01))
noisefunc$y <- runif(nrow(noisefunc), min=-1, max=1)
noisefunc <- noisefunc[sample(nrow(noisefunc)),]
noisefunc_train <- noisefunc[1:100,]
noisefunc_val   <- noisefunc[101:150,]
noisefunc_test  <- noisefunc[151:201,]
#

# funció determinista + soroll ~ N(0,0.5), variables fora dels rangs recomanats per XN
satfunc <- data.frame(x=seq(0, pi, by=0.01))
satfunc$y <- satfunc$x * sin(5*satfunc$x) + rnorm(length(satfunc$x), sd=0.2)
satfunc <- satfunc[sample(nrow(satfunc)),]
satfunc_train <- satfunc[1:200,]
satfunc_val   <- satfunc[201:250,]
satfunc_test  <- satfunc[251:315,]
#


################################################################################
# Carregar datasets externs
################################################################################

building1 <- read.table("data/building1.dt", skip=7, header=FALSE)
building1_train <- building1[1:2104,]     # train, primeres 2104 instàncies
building1_val   <- building1[2105:3156,]  # validació, seguents 1052
building1_test  <- building1[3157:4208,]  # test, últimes 1052

building2 <- read.table("data/building2.dt", skip=7, header=FALSE)
building2_train <- building2[1:2104,]     # idem anterior
building2_val   <- building2[2105:3156,]
building2_test  <- building2[3157:4208,]

building3 <- read.table("data/building3.dt", skip=7, header=FALSE)
building3_train <- building3[1:2104,]     # idem anterior
building3_val   <- building3[2105:3156,]
building3_test  <- building3[3157:4208,]


flare1 <- read.table("data/flare1.dt", skip=7, header=FALSE)
flare1_train <- flare1[1:533,]      # train, primeres 533 instàncies
flare1_val   <- flare1[534:800,]    # validació, següents 267
flare1_test  <- flare1[801:1066,]   # test, últimes 266

flare2 <- read.table("data/flare2.dt", skip=7, header=FALSE)
flare2_train <- flare2[1:533,]      # idem anterior
flare2_val   <- flare2[534:800,]
flare2_test  <- flare2[801:1066,]

flare3 <- read.table("data/flare3.dt", skip=7, header=FALSE)
flare3_train <- flare3[1:533,]      # idem anterior
flare3_val   <- flare3[534:800,]
flare3_test  <- flare3[801:1066,]


card1 <- read.table("data/card1.dt", skip=7, header=FALSE)
card1_train <- card1[1:345,]
card1_val   <- card1[346:518,]
card1_test  <- card1[519:690,]

card2 <- read.table("data/card2.dt", skip=7, header=FALSE)
card2_train <- card2[1:345,]
card2_val   <- card2[346:518,]
card2_test  <- card2[519:690,]

card3 <- read.table("data/card3.dt", skip=7, header=FALSE)
card3_train <- card3[1:345,]
card3_val   <- card3[346:518,]
card3_test  <- card3[519:690,]


heart1 <- read.table("data/heart1.dt", skip=7, header=FALSE)
heart1_train <- heart1[1:460,]
heart1_val   <- heart1[461:690,]
heart1_test  <- heart1[691:920,]

heart2 <- read.table("data/heart2.dt", skip=7, header=FALSE)
heart2_train <- heart2[1:460,]
heart2_val   <- heart2[461:690,]
heart2_test  <- heart2[691:920,]

heart3 <- read.table("data/heart3.dt", skip=7, header=FALSE)
heart3_train <- heart3[1:460,]
heart3_val   <- heart3[461:690,]
heart3_test  <- heart3[691:920,]


thyroid1 <- read.table("data/thyroid1.dt", skip=7, header=FALSE)
thyroid1_train <- thyroid1[1:3600,]
thyroid1_val   <- thyroid1[3601:5400,]
thyroid1_test  <- thyroid1[5401:7200,]

thyroid2 <- read.table("data/thyroid2.dt", skip=7, header=FALSE)
thyroid2_train <- thyroid2[1:3600,]
thyroid2_val   <- thyroid2[3601:5400,]
thyroid2_test  <- thyroid2[5401:7200,]

thyroid3 <- read.table("data/thyroid3.dt", skip=7, header=FALSE)
thyroid3_train <- thyroid3[1:3600,]
thyroid3_val   <- thyroid3[3601:5400,]
thyroid3_test  <- thyroid3[5401:7200,]


################################################################################
# Funcions d'utilitat
################################################################################

#
# Calcula el rank de Pareto de les files d'una matriu numèrica.
#
paretorank <- function(m)
{
    ranks <- 1:nrow(m)
    
    for (i in 1:nrow(m))
    {
        obs <- m[i,]
        ranks[i] <- 0
        
        for (j in 1:nrow(m))
        {
            if (i != j)
            {
                alt <- m[j,]
                nod <- TRUE
                
                for (k in 1:ncol(m))
                    if (obs[k] < alt[k]) nod <- FALSE
                
                if (nod) ranks[i] <- ranks[i] + 1
            }
        }
    }
    
    return(ranks)
}

#
# Avalua una xarxa neural, realitzant 10 entrenaments, i calculant l'MSE promig
# sobre els conjunts train, val, test. Retorna llista amb els resultats.
#
ann_eval <- function(name, train, val, test, inputs, outputs, lact, hls, lr, mo, tmse)
{
    train_mse_vec <- NULL
    val_mse_vec   <- NULL
    test_mse_vec  <- NULL
    numepochs_vec <- NULL

    for (i in 1:10)
    {
        model <- ann(train[inputs], train[outputs], hls=hls, lr=lr, mo=mo, tmse=tmse, lact=lact)

        fitted <- predict(model, newdata=train[inputs])
        errors <- train[outputs] - fitted
        train_mse = sum(errors^2) / (nrow(errors) * ncol(errors))  # normalitzar segons #exemples i #sortides

        fitted <- predict(model, newdata=val[inputs])
        errors <- val[outputs] - fitted
        val_mse = sum(errors^2) / (nrow(errors) * ncol(errors))

        fitted <- predict(model, newdata=test[inputs])
        errors <- test[outputs] - fitted
        test_mse = sum(errors^2) / (nrow(errors) * ncol(errors))
        
        train_mse_vec <- c(train_mse_vec, train_mse)
        val_mse_vec   <- c(val_mse_vec, val_mse)
        test_mse_vec  <- c(test_mse_vec, test_mse)
        numepochs_vec <- c(numepochs_vec, model$num_epochs)
    }
    
    r = list(
        name,
        mean(train_mse_vec) * 100,
        sd(train_mse_vec) * 100,
        mean(val_mse_vec) * 100,
        sd(val_mse_vec) * 100,
        mean(test_mse_vec) * 100,
        sd(test_mse_vec) * 100,
        mean(numepochs_vec),
        sd(numepochs_vec)
        )
    
    return(r)
}

#
# Realitza un report de l'evolució de l'algorisme genètic sobre un dataset.
#
gann_report <- function(name, train, val, test, inputs, outputs, lact, stophs=14, npf=0.01/1E4, nef=0.01/1E4)
{
    print(paste("***** DATASET:", name))
    
    evol <- gann(train[inputs], train[outputs], val[inputs], val[outputs],
                 lact=lact, stophs=stophs, npf=npf, nef=nef)
    
    annresult <- ann_eval(name, train, val, test, inputs, outputs, lact,
                          evol$besthls, evol$bestlr, evol$bestmo, evol$besttmse)
    
    plot(evol, main=name, xlab="generació", ylab="fitness")
    
    hiddens <- paste(evol$besthls, collapse="-")
    topology <- paste(length(inputs), hiddens, length(outputs), sep="-")
    
    evolresults <<- rbind(evolresults, list(name, hiddens, topology, evol$bestlr, evol$bestmo, evol$besttmse))
    annresults <<- rbind(annresults, annresult)
}

#
# Genera i guarda en un fitxer, la observació d'una execució de l'AG,
# per a un dataset i configuració donats.
#
ganntunning_mkobs <- function(trainset, valset, inputs, outputs, lact, of, el, sb, cp, mp)
{
    fv <- NULL
    lv <- NULL

    datasetname = deparse(substitute(trainset))
    fname = sprintf("gann_tunning_data/%s_%d_%d_%d_%3.2f_%3.2f.Rdata", datasetname, of, el, sb, cp, mp)

    for (i in 1:3)
    {
        print(paste("***** DATASET:", datasetname, "RUN:", i, "CONFIG:", of, el, sb, cp, mp))
        print(fname)
        
        r <- gann(trainset[inputs], trainset[outputs], valset[inputs], valset[outputs],
                  offspring=of, elitism=el, sbeta=sb, cp=cp, mp=mp,
                  lact=lact, maxge=200, maxep=1000)
        
        fv <- c(fv, tail(r$maxfitness, 1))
        lv <- c(lv, r$lifespancount)
    }
    
    f <- mean(fv)
    l <- mean(lv)
    
    obs <- list(datasetname, of, el, sb, cp, mp, f, l)  # la llista admet tipus diferents
    save(obs, file=fname)
}

#
# Genera una mostra d'observacions per a diferents datasets i una configuració donada.
# Per defecte s'executa amb la configuració base (veure la memòria del treball).
#
ganntunning_sample <- function(of=16, el=3, sb=4, cp=0.95, mp=0.05)
{
    # dades sintètiques
    ganntunning_mkobs(sinfunc_train,   sinfunc_test,  1,    2,     "tanh",    of, el, sb, cp, mp)
    ganntunning_mkobs(discfunc_train,  discfunc_test, 1,    2,     "tanh",    of, el, sb, cp, mp)
    
    # regressions de PROBEN1
    ganntunning_mkobs(building1_train, building1_val, 1:14, 15:17, "sigmoid", of, el, sb, cp, mp)
    ganntunning_mkobs(flare1_train,    flare1_val,    1:24, 25:27, "sigmoid", of, el, sb, cp, mp)
    
    # classificacions de PROBEN1
    ganntunning_mkobs(card1_train,     card1_val,     1:51, 52:53, "sigmoid", of, el, sb, cp, mp)
    ganntunning_mkobs(heart1_train,    heart1_val,    1:35, 36:37, "sigmoid", of, el, sb, cp, mp)
    ganntunning_mkobs(thyroid1_train,  thyroid1_val,  1:21, 22:24, "sigmoid", of, el, sb, cp, mp)
}

#
# Carrega dades MNIST, convertint la sortida categòrica dels 10 dígits,
# en 10 sortides binàries amb varors -0.9 (no), 0.9 (si) per a cada dígit.
#
readMNIST <- function(imgfilename, lblfilename)
{
    imgfile <- gzfile(imgfilename, "rb")
    lblfile <- gzfile(lblfilename, "rb")
    
    img_h <- readBin(imgfile, integer(), n=4, endian="big")
    lbl_h <- readBin(lblfile, integer(), n=2, endian="big")
    
    numinstances <- img_h[[2]]
    width <- img_h[[3]]
    height <- img_h[[4]]
    
    img_data <- readBin(imgfile, integer(), size=1, n=width*height*numinstances, endian="big", signed=FALSE)
    lbl_data <- readBin(lblfile, integer(), size=1, n=numinstances, endian="big")
    
    close(lblfile)
    close(imgfile)
    
    digits <- matrix(img_data, ncol=width*height, nrow=numinstances, byrow=TRUE)
    digits <- digits / 128 - 1  # normalitzar a [-1, 1]
    
    labels <- matrix(data=-0.9, ncol=10, nrow=numinstances)
    colnames(labels) <- c("L0", "L1", "L2", "L3", "L4", "L5", "L6", "L7", "L8", "L9")
    for (i in 1:numinstances)
    {
        j <- lbl_data[[i]] + 1
        labels[i,j] <- 0.9
    }

    return(data.frame(digits, labels))
}



