#
# Script que recrea la demo de la presentació
#

#install.packages("../genann", repos=NULL)
library("genann")

set.seed(2016)  # intentar produir el mateix resultat, pot dependre
                # del generador de nombres aleatoris del sistema

# generar un dataset sintètic
data_source <- data.frame(x=seq(-2, 2, by=0.004))
data_source$y <- exp(-data_source$x^2) * sin(pi*data_source$x)

# afegir soroll a les dades font
data <- data.frame(x=data_source$x, y=data_source$y + rnorm(nrow(data_source), sd=0.6))
data <- data[sample(nrow(data)),]  # desordenar les files aleatoriament

# partir en conjunts d'entrenament i prova
data_train <- data[1:700,]       # 70% entrenament XN i AG
data_test  <- data[701:1001,]    # 30% validació del model

# partir subconjunt d'entrenament en 2 per l'AG
data_gt <- data_train[1:350,]    # 50% entrenament per l'AG
data_gv <- data_train[351:700,]  # 50% validació per l'AG


# executar algorisme genètic
evol <- gann(data_gt$x, data_gt$y, data_gv$x, data_gv$y, lact="tanhtt")

# crear el model predictiu amb la xarxa neural configurada per l'AG
model <- ann(data_train$x, data_train$y, lact="tanhtt", tmse=evol$besttmse, lr=evol$bestlr, mo=evol$bestmo, hls=evol$besthls)


fitted <- predict(model, newdata=data_test$x)  # realitzar prediccions amb el conjunt test
errors <- data_test$y - fitted                 # residus
print(sum(errors^2) / nrow(errors))            # error quadràtic mig

save(data_source, data, data_train, data_test, evol, model, fitted, errors, file="datapr_results.Rdata")


